/*
 * Board.java
 * 
 * $Id: Board.java,v 1.8 2012-04-14 06:17:21 jbf9160 Exp $
 * 
 * $Log: Board.java,v $
 * Revision 1.8  2012-04-14 06:17:21  jbf9160
 * Small things adjusted to get everything to work. GoodPlayer algorithm implemented. Finished source.
 *
 * Revision 1.7  2012-04-14 04:08:02  jbf9160
 * Implemented a getPieceAt method that lets you "look" at the board. Implemented due to needed information in the GoodPlayer class.
 *
 * Revision 1.6  2012-04-14 03:05:22  jbf9160
 * Commented on some points that explain my reasoning if other programmers read in future. Created new method in Player class to remove acouple lines of repeated code.
 *
 * Revision 1.5  2012-04-14 02:44:11  jbf9160
 * While GoodPlayer still needs an algorithm, the Board class' method to check to see if someone has won the game has been implemented and checked for completeness. Various other bugs and updates have been added to the Player classes including a modified nextMove that contains all the necessary methods to update the board to reduce clutter in the main Connect4 method.
 *
 * Revision 1.4  2012-04-14 00:32:18  jbf9160
 * Small changes to methods. Added while (board is not full) loop to execute nextMove methods then update the board.
 *
 * Revision 1.3  2012-04-11 01:31:55  jbf9160
 * Completed the BadPlayer class. Added methods to Board class to determine if the board is full along with fullColumn and emptyColumn methods.
 *
 * Revision 1.2  2012-04-11 00:46:42  jbf9160
 * All players implemented as children of Player class. Player.nextMove only returns 0 for the sake of testing.
 *
 * Revision 1.1  2012-04-10 23:17:49  jbf9160
 * Currently containing errors due to classes I have not yet implemented. Created Board class with newPlayer and printBoard methods
 *
 */

/**
 * Board class holds all board information and methods
 * 
 * @author James Fator (jbf9160@rit.edu)
 */
public class Board {

	public final Player playerX;
	public final Player playerO;
	private final int R;
	private final int C;
	private String[][] table;
	private String lastPiece;
	private int lastColumn;
	
	/**
	 * Constructor for the Board class.
	 * Fills the instance variables then creates a board based on RxC size.
	 * 
	 * @param args player-X player-O [#rows #cols]
	 */
	public Board(String[] args) {
		this.playerX = newPlayer("X", args[0]);
		this.playerO = newPlayer("O", args[1]);
		this.lastPiece = " ";
		this.lastColumn = 0;
		int XR, XC;
		try {
			XR = Integer.parseInt(args[2]);
			XC = Integer.parseInt(args[3]);
		} catch (Exception e) {
			XC = XR = 4;
		}
		this.R = XR;
		this.C = XC;
		table = new String[C*2+1][R+1];
		for (int r = 0; r < R+1; r++) {
			for (int c = 0; c < C*2+1; c++) {
				if (c % 2 == 0) {
					if (r == 0) {
						table[c][r] = "+";
					} else {
						table[c][r] = "|";
					}
				} else {
					if (r == 0) {
						table[c][r] = "-";
					} else {
						table[c][r] = " ";
					}
				}
			}
		}
	}
	
	/**
	 * getRows returns the number of rows in the game
	 * 
	 * @return number of rows in the game
	 */
	public int getRows() {
		return this.R;
	}
	
	/**
	 * getCol returns the number of columns in the game
	 * 
	 * @return number of columns in the game
	 */
	public int getCol() {
		return this.C;
	}
	
	/**
	 * endGame determines if one player has 4 tiles in a row.
	 * 
	 * @return true if the game is over
	 */
	public boolean endGame() {
		int c;
		int r;
		int tX = 0;
		int tO = 0;
		// check columns for 4 consecutive pieces
		for (c = 1; c < this.C*2; c += 2) {
			for (r = 1; r <= this.R; r++) {
				if (this.table[c][r].equals("X")) {
					tX++;
					tO = 0;
				} else if (this.table[c][r].equals("O")) {
					tO++;
					tX = 0;
				}
				if (tX == 4) {
					System.out.println("X won in column " + (c-1)/2);
					return true;
				} else if (tO == 4) {
					System.out.println("O won in column " + (c-1)/2);
					return true;
				}
			}
			tX = 0;
			tO = 0;
		}
		// check rows for 4 consecutive pieces
		for (r = 1; r <= this.R; r++) {
			for (c = 1; c < this.C*2; c += 2) {
				if (this.table[c][r].equals("X")) {
					tX++;
					tO = 0;
				} else if (this.table[c][r].equals("O")) {
					tO++;
					tX = 0;
				} else {
					tX = 0;
					tO = 0;
				}
				if (tX == 4) {
					System.out.println("X won in row " + (this.R-r));
					return true;
				} else if (tO == 4) {
					System.out.println("O won in row " + (this.R-r));
					return true;
				}
			}
			tX = 0;
			tO = 0;
		}
		// check back-slash diagonal for 4 consecutive pieces
		for (c = 7; c < this.C*2; c += 2) {
			for (r = 1; r < this.R-2; r++) {
				if (!table[c][r].equals(" ") && 
						table[c][r].equals(table[c-2][r+1]) &&
						table[c][r].equals(table[c-4][r+2]) &&
						table[c][r].equals(table[c-6][r+3])) {
					System.out.println(table[c][r]+" won in a diagonal");
					return true;
				}
			}
		}
		// check forward-slash diagonal for 4 consecutive pieces
		for (c = 1; c < this.C*2-6; c += 2) {
			for (r = 1; r < this.R-2; r++) {
				if (!table[c][r].equals(" ") && 
						table[c][r].equals(table[c+2][r+1]) &&
						table[c][r].equals(table[c+4][r+2]) &&
						table[c][r].equals(table[c+6][r+3])) {
					System.out.println(table[c][r]+" won in a diagonal");
					return true;
				}
			}
		}
		// no winner found, return hanging your head in shame
		// and with a false-valued boolean.
		return false;
	}
	
	/**
	 * printBoard prints the current Connect4 game board
	 */
	public void printBoard() {
		System.out.println("");
		for (int r = this.R; r >= 0; r--) {
			for (int c = 0; c < this.C*2+1; c++) {
				System.out.print(this.table[c][r]);
			}
			System.out.println("");
		}
		System.out.println("");
	}
	
	/**
	 * getPieceAt simulates looking at the game board by returning
	 * the piece that would be at a specific row or column
	 * 
	 * @param r row index
	 * @param c column index
	 * @return " ", "X", or "O" depending on which piece is at RxC
	 */
	public String getPieceAt(int r, int c){
		return this.table[c][r];
	}
	
	/**
	 * newPlayer returns an instance of a Player type based on the
	 * String argument
	 * 
	 * @param type argument String equaling the name of a Player 
	 *        implemented class
	 *        
	 * @return a Player implemented class
	 */
	private Player newPlayer(String name, String type) {
        if (type.equals("human")) {
            return new HumanPlayer(name);
        } else if (type.equals("bad")) {
            return new BadPlayer(name);
        } else if (type.equals("good")) {
            return new GoodPlayer(name);
        } else {
            return new RandomPlayer(name);
        }
    }
	
	/**
	 * updateBoard updates the current board data structure
	 * 
	 * @param column which to place move
	 * @param name of player
	 * @return $column if everything checks out
	 * @return -2 if the board is full
	 * @return -3 if there is a winner
	 */
	public int updateBoard(int column, String name) {
		for (int r = 1; r < this.R+1; r++) {
			if (this.table[column][r].equals(" ")) {
				this.table[column][r] = name;
				this.lastPiece = name;
				this.lastColumn = column;
				break;
			}
		}
		// we're doing the checks here so we don't have to repeat in
		// the main method.
		if (this.fullBoard()) {
			return -2;
		} else if (this.endGame()) {
			return -3;
		} else {
			return column;
		}
	}
	
	/**
	 * @return lastPiece to be played
	 */
	public String getLastPiece() {
		return this.lastPiece;
	}
	
	/**
	 * @return lastColumn to be played
	 */
	public int getLastColumn() {
		return this.lastColumn;
	}
	
	/**
	 * emptyColumn checks a single column at int column to see if no 
	 * spots in that column are empty
	 * 
	 * @param column number
	 * 
	 * @return true if column is empty
	 */
	public boolean emptyColumn(int column) {
		if (this.table[column][1].equals(" ")) {
			return true;
		} else {
			return false;
		}
	}
	
	/**
	 * fullColumn checks a single column at int column to see if all spots
	 * are taken
	 * 
	 * @param column number
	 * 
	 * @return true if the column is full
	 */
	public boolean fullColumn(int column) {
		for (int r = 1; r < this.R+1; r++) {
			if (this.table[column][r].equals(" ")) {
				return false;
			}
		}
		return true;
	}
	
	/**
	 * fullBoard checks to see if all table locations are free
	 * 
	 * @return true if the board if full
	 */
	public boolean fullBoard() {
		for (int c = 1; c < this.C*2+1; c += 2) {
			if (! this.fullColumn(c)) {
				return false;
			}
		}
		System.out.println("Its a tie, no one wins");
		return true;
	}
}
