/*
 * Connect4.java
 * 
 * $Id: Connect4.java,v 1.7 2012-04-16 00:45:26 jbf9160 Exp $
 * 
 * $Log: Connect4.java,v $
 * Revision 1.7  2012-04-16 00:45:26  jbf9160
 * Small updates to match the output check with the try command.
 *
 * Revision 1.6  2012-04-14 06:17:20  jbf9160
 * Small things adjusted to get everything to work. GoodPlayer algorithm implemented. Finished source.
 *
 * Revision 1.5  2012-04-14 03:05:21  jbf9160
 * Commented on some points that explain my reasoning if other programmers read in future. Created new method in Player class to remove acouple lines of repeated code.
 *
 * Revision 1.4  2012-04-14 02:44:11  jbf9160
 * While GoodPlayer still needs an algorithm, the Board class' method to check to see if someone has won the game has been implemented and checked for completeness. Various other bugs and updates have been added to the Player classes including a modified nextMove that contains all the necessary methods to update the board to reduce clutter in the main Connect4 method.
 *
 * Revision 1.3  2012-04-14 00:32:18  jbf9160
 * Small changes to methods. Added while (board is not full) loop to execute nextMove methods then update the board.
 *
 * Revision 1.2  2012-04-11 00:04:50  jbf9160
 * Committing before implementation of Player classes. Modified Connect4.invalidArguments method to look nicer and give a better error description for debugging.
 *
 * Revision 1.1  2012-04-10 22:27:37  jbf9160
 * Created the Connect4 class. I have only implemented the argument checking method so far.
 *
 */

/**
 * Connect4 is the main class for the Connect 4 game project.
 * 
 * @author James Fator (jbf9160@rit.edu)
 */
public class Connect4 {
	
	/**
	 * main method reads arguments to create players and board
	 * 
	 * @param args player-X player-O [#rows #cols]
	 */
	public static void main(String[] args) {
		if (incorrectArguments(args)) {
			return;
		}
		Board board = new Board(args);
		int n = playGame(board);
		// There is absolutely no use for this switch besides giving
		// a heads up to how the game ended. I just wanted to dabble
		// in the realm of messing with boolean-like integers.
		switch (n) {
			case -1: // human player quit
			case -2: // tie game
			case -3: // someone wins!
		}
	}
	
	/**
	 * playGame is where the magic happens:
	 *     the board is printed then the method goes into a loop
	 *     where the Player's nextMove methods are being called until
	 *     either the game ends, the board becomes full or the human
	 *     issues an exit command (-1).
	 *     
	 * @param board the gameboard
	 * 
	 * @return an integer n that denotes how the game ended.
	 */
	private static int playGame(Board board) {
		board.printBoard();
		int n;
		while (true) {
			//playerX's turn
			board.playerX.playerMoving();
			n = board.playerX.nextMove(board);
			board.printBoard();
			if (n < 0) {
				return n;
			}
			//playerO's turn
			board.playerO.playerMoving();
			n = board.playerO.nextMove(board);
			board.printBoard();
			if (n < 0) {
				return n;
			}
		}
	}
	
	/**
	 * incorrectArguments checks to see if all the arguments are use-able
	 * 
	 * @param args player-X player-O [#rows #cols]
	 * 
	 * @return true if the arguments are incorrect
	 */
	private static boolean incorrectArguments(String[] args) {
		String human = "human";
		String bad = "bad";
		String good = "good";
		String random = "random";
		try {
			if (args.length == 0 || !(args.length == 2 || args.length == 4)) {
				throw new Exception("Invalid number of arguments.");
			} else if (! (args[0].equals(human) || args[0].equals(bad)
					|| args[0].equals(good) || args[0].equals(random))) {
				throw new Exception("Invalid player X type.");
			} else if (! (args[1].equals(human) || args[1].equals(bad)
					|| args[1].equals(good) || args[1].equals(random))) {
				throw new Exception("Invalid player O type.");
			} else if (args.length == 4
					&& (Integer.parseInt(args[2]) < 1 || Integer
							.parseInt(args[2]) > 6)) {
				throw new Exception("Invalid row value.");
			} else if (args.length == 4
					&& (Integer.parseInt(args[3]) < 1 || Integer
							.parseInt(args[3]) > 7)) {
				throw new Exception("Invalid column value.");
			} else {
				return false;
			}
		} catch (Exception e) {
			//System.err.println(e);
			System.err.println("Usage: java Connect4 player-X player-O "
					+ "[#rows #cols]\nwhere player-X and player-O are one "
					+ "of: human bad good random\n[#rows #cols] are optional,"
					+ " if provided their values must be\nin the ranges: "
					+ "#rows from 1 to 6 and #cols from 1 to 7");
			return true;
		}
	}
}
