/*
 * GoodPlayer.java
 * 
 * $Id: GoodPlayer.java,v 1.5 2012-04-14 06:17:22 jbf9160 Exp $
 * 
 * $Log: GoodPlayer.java,v $
 * Revision 1.5  2012-04-14 06:17:22  jbf9160
 * Small things adjusted to get everything to work. GoodPlayer algorithm implemented. Finished source.
 *
 * Revision 1.4  2012-04-14 03:05:22  jbf9160
 * Commented on some points that explain my reasoning if other programmers read in future. Created new method in Player class to remove acouple lines of repeated code.
 *
 * Revision 1.3  2012-04-14 02:44:11  jbf9160
 * While GoodPlayer still needs an algorithm, the Board class' method to check to see if someone has won the game has been implemented and checked for completeness. Various other bugs and updates have been added to the Player classes including a modified nextMove that contains all the necessary methods to update the board to reduce clutter in the main Connect4 method.
 *
 * Revision 1.2  2012-04-11 01:31:55  jbf9160
 * Completed the BadPlayer class. Added methods to Board class to determine if the board is full along with fullColumn and emptyColumn methods.
 *
 * Revision 1.1  2012-04-11 00:46:42  jbf9160
 * All players implemented as children of Player class. Player.nextMove only returns 0 for the sake of testing.
 *
 */

/**
 * GoodPlayer class extends the Player class and is used as a
 * module in the Connect4 Game.
 * 
 * @author James Fator (jbf9160@rit.edu)
 */
public class GoodPlayer extends Player {
	
	/**
	 * Constructor for the GoodPlayer Class.
	 * Simply calls the constructor from the super class with an added
	 * type argument.
	 * 
	 * @param name X or O.
	 */
	public GoodPlayer(String name) {
		super(name, "good");
	}
	
	/**
	 * nextMove decides the next move for the player.
	 * 
	 * @return an integer for which column to place move.
	 */
	public int nextMove(Board board) {
		// checks to see if there are 3 in a row anywhere
		int j = this.endGame(board);
		if (j != -1) {
			return super.nextMove(board, j);
		} else if (!board.fullColumn(board.getCol())) {
			// check to see if we can put it in the center
			return super.nextMove(board, board.getCol());
		} else if (!board.fullColumn(board.getLastColumn())) {
			// check to see if we can put it on top of the last piece
			return super.nextMove(board, board.getLastColumn());
		}
		// if all else fails...
		for (int c = 1; c < board.getCol()*2; c += 2) {
			if (!board.fullColumn(c)) {
				return super.nextMove(board, c);
			}
		}
		// Impossible to get to this point.
		return -1;
	}
	
	/**
	 * modified endGame determines if one player has 3 tiles in a row.
	 * brought over to BadPlayer for strategy purposes
	 * 
	 * @return true if the game will be over
	 */
	public int endGame(Board board) {
		int c;
		int r;
		int tX = 0;
		int tO = 0;
		// check columns for 3 consecutive pieces
		for (c = 1; c < board.getCol()*2; c += 2) {
			for (r = 1; r <= board.getRows(); r++) {
				if (board.getPieceAt(r, c).equals("X")) {
					tX++;
					tO = 0;
				} else if (board.getPieceAt(r, c).equals("O")) {
					tO++;
					tX = 0;
				}
				if (tX == 3 && board.getRows() > r++ && 
						board.getPieceAt(r++, c).equals(" ")) {
					return c;
				} else if (tO == 3 && board.getRows() > r++ && 
						board.getPieceAt(r++, c).equals(" ")) {
					return c;
				}
			}
			tX = 0;
			tO = 0;
		}
		// check rows for 3 consecutive pieces
		for (r = 1; r <= board.getRows(); r++) {
			for (c = 1; c < board.getCol()*2; c += 2) {
				if (board.getPieceAt(r, c).equals("X")) {
					tX++;
					tO = 0;
				} else if (board.getPieceAt(r, c).equals("O")) {
					tO++;
					tX = 0;
				} else {
					tX = 0;
					tO = 0;
				}
				if (tX == 3 && 0 < c-2 && 
						board.getPieceAt(r, c-2).equals(" ")) {
					return c-2;
				} else if (tO == 3 && board.getCol()*2 > c+2 && 
						board.getPieceAt(r, c+2).equals(" ")) {
					return c+2;
				}
			}
			tX = 0;
			tO = 0;
		}
		// check back-slash diagonal for 3 consecutive pieces
		for (c = 7; c < board.getCol()*2; c += 2) {
			for (r = 1; r < board.getRows()-2; r++) {
				if (!board.getPieceAt(r, c).equals(" ") && 
						board.getPieceAt(r, c).equals
						(board.getPieceAt(r+1, c-2)) &&
						board.getPieceAt(r, c).equals
						(board.getPieceAt(r+2, c-4)) &&
						board.getPieceAt(r+3, c-6).equals(" ")) {
					return c-6;
				}
			}
		}
		// check forward-slash diagonal for 3 consecutive pieces
		for (c = 1; c < board.getCol()*2-6; c += 2) {
			for (r = 1; r < board.getRows()-2; r++) {
				if (!board.getPieceAt(r, c).equals(" ") && 
						board.getPieceAt(r, c).equals
						(board.getPieceAt(r+1, c+2)) &&
						board.getPieceAt(r, c).equals
						(board.getPieceAt(r+2, c+4)) &&
						board.getPieceAt(r+3, c+6).equals(" ")) {
					return c+6;
				}
			}
		}
		return -1;
	}

}