/*
 * HumanPlayer.java
 * 
 * $Id: HumanPlayer.java,v 1.7 2012-04-16 00:45:26 jbf9160 Exp $
 * 
 * $Log: HumanPlayer.java,v $
 * Revision 1.7  2012-04-16 00:45:26  jbf9160
 * Small updates to match the output check with the try command.
 *
 * Revision 1.6  2012-04-14 06:17:20  jbf9160
 * Small things adjusted to get everything to work. GoodPlayer algorithm implemented. Finished source.
 *
 * Revision 1.5  2012-04-14 03:05:22  jbf9160
 * Commented on some points that explain my reasoning if other programmers read in future. Created new method in Player class to remove acouple lines of repeated code.
 *
 * Revision 1.4  2012-04-14 02:44:11  jbf9160
 * While GoodPlayer still needs an algorithm, the Board class' method to check to see if someone has won the game has been implemented and checked for completeness. Various other bugs and updates have been added to the Player classes including a modified nextMove that contains all the necessary methods to update the board to reduce clutter in the main Connect4 method.
 *
 * Revision 1.3  2012-04-11 02:47:40  jbf9160
 * Finished HumanPlayer class. Added comments to Player Class.
 *
 * Revision 1.2  2012-04-11 01:31:54  jbf9160
 * Completed the BadPlayer class. Added methods to Board class to determine if the board is full along with fullColumn and emptyColumn methods.
 *
 * Revision 1.1  2012-04-11 00:46:42  jbf9160
 * All players implemented as children of Player class. Player.nextMove only returns 0 for the sake of testing.
 *
 */

/**
 * HumanPlayer class extends the Player class and is used as a
 * module in the Connect4 Game.
 * 
 * @author James Fator (jbf9160@rit.edu)
 */
public class HumanPlayer extends Player {
	
	/**
	 * Constructor for the HumanPlayer Class.
	 * Simply calls the constructor from the super class with an added
	 * type argument.
	 * 
	 * @param name X or O.
	 */
	public HumanPlayer(String name) {
		super(name, "human");
	}
	
	/**
	 * nextMove decides the next move for the player.
	 * 
	 * @return an integer for which column to place move.
	 */
	public int nextMove(Board board) {
		int c;
		while (true) {
			System.out.print("Player " + super.getName() + ": Enter the column to " +
					"drop your piece (-1 to quit): ");
			c = super.inputObject().nextInt() * 2 + 1;
			if (c == -1) {
				System.out.println(super.getName() + " quits the game");
				return -1;
			} else if (c < board.getCol() * 2 + 1 && c > 0
					&& !board.fullColumn(c)) {
				break;
			} else {
				System.out.println("invalid column: " + (c-1)/2);
			}
		}
		return super.nextMove(board, c);
	}

}
