/*
 * Player.java
 * 
 * $Id: Player.java,v 1.7 2012-04-14 06:17:21 jbf9160 Exp $
 * 
 * $Log: Player.java,v $
 * Revision 1.7  2012-04-14 06:17:21  jbf9160
 * Small things adjusted to get everything to work. GoodPlayer algorithm implemented. Finished source.
 *
 * Revision 1.6  2012-04-14 03:05:22  jbf9160
 * Commented on some points that explain my reasoning if other programmers read in future. Created new method in Player class to remove acouple lines of repeated code.
 *
 * Revision 1.5  2012-04-14 02:44:11  jbf9160
 * While GoodPlayer still needs an algorithm, the Board class' method to check to see if someone has won the game has been implemented and checked for completeness. Various other bugs and updates have been added to the Player classes including a modified nextMove that contains all the necessary methods to update the board to reduce clutter in the main Connect4 method.
 *
 * Revision 1.4  2012-04-14 00:32:18  jbf9160
 * Small changes to methods. Added while (board is not full) loop to execute nextMove methods then update the board.
 *
 * Revision 1.3  2012-04-11 02:47:40  jbf9160
 * Finished HumanPlayer class. Added comments to Player Class.
 *
 * Revision 1.2  2012-04-11 01:31:55  jbf9160
 * Completed the BadPlayer class. Added methods to Board class to determine if the board is full along with fullColumn and emptyColumn methods.
 *
 * Revision 1.1  2012-04-11 00:46:42  jbf9160
 * All players implemented as children of Player class. Player.nextMove only returns 0 for the sake of testing.
 *
 */

import java.util.Scanner;

/**
 * Player is an abstract class that contains methods for all the
 * different player types.
 * 
 * @author James Fator (jbf9160@rit.edu)
 */
public abstract class Player {
	
	private static Scanner keyboardInput = new Scanner(System.in);
	private final String playerType;
	private final String playerName;
	
	/**
	 * Constructor for Player class.
	 * 
	 * @param name of move (X or O)
	 * @param type of player
	 */
	public Player(String name, String type) {
		this.playerName = name;
		this.playerType = type;
	}
	
	/**
	 * getName returns the player's name.
	 * 
	 * @return X or O
	 */
	public String getName() {
		return this.playerName;
	}
	
	/**
	 * getType returns the player's type.
	 * 
	 * @return player's type
	 */
	public String getType() {
		return this.playerType;
	}
	
	/**
	 * inputObject returns instance of Scanner.
	 * 
	 * @return Scanner object
	 */
	public Scanner inputObject() {
		return Player.keyboardInput;
	}
	
	/**
	 * playerMoving prints the phrase:
	 *     "$playerType player $playerName moving..."
	 */
	public void playerMoving() {
		System.out.println(this.playerType+" player " + 
	        this.playerName + " moving...");
		return;
	}
	
	/**
	 * playerMoved prints the phrase:
	 *     "Player drops an $name piece into column: $column"
	 *     
	 * @param column
	 */
	public void playerMoved(int column) {
		System.out.println("Player drops an " + this.playerName + 
				" piece into column: "+(column-1)/2);
		return;
	}
	
	public abstract int nextMove(Board board);
	
	/**
	 * nextMove with parameters c was implemented to remove repeating code
	 *     from child classes by moving extra method calls to one
	 *     unified method
	 *     
	 * @param board the game board
	 * @param c column at which to place the piece
	 * 
	 * @return column at which the piece was placed or the end game notifier
	 */
	public int nextMove(Board board, int c){
		this.playerMoved(c);
		return board.updateBoard(c, this.playerName);
	}

}
