/*
 * RandomPlayer.java
 * 
 * $Id: RandomPlayer.java,v 1.5 2012-04-14 03:05:21 jbf9160 Exp $
 * 
 * $Log: RandomPlayer.java,v $
 * Revision 1.5  2012-04-14 03:05:21  jbf9160
 * Commented on some points that explain my reasoning if other programmers read in future. Created new method in Player class to remove acouple lines of repeated code.
 *
 * Revision 1.4  2012-04-14 02:44:11  jbf9160
 * While GoodPlayer still needs an algorithm, the Board class' method to check to see if someone has won the game has been implemented and checked for completeness. Various other bugs and updates have been added to the Player classes including a modified nextMove that contains all the necessary methods to update the board to reduce clutter in the main Connect4 method.
 *
 * Revision 1.3  2012-04-11 01:46:04  jbf9160
 * RandomPlayer module completed.
 *
 * Revision 1.2  2012-04-11 01:31:54  jbf9160
 * Completed the BadPlayer class. Added methods to Board class to determine if the board is full along with fullColumn and emptyColumn methods.
 *
 * Revision 1.1  2012-04-11 00:46:42  jbf9160
 * All players implemented as children of Player class. Player.nextMove only returns 0 for the sake of testing.
 *
 */

import java.util.Random;

/**
 * RandomPlayer class extends the Player class and is used as a
 * module in the Connect4 Game.
 * 
 * @author James Fator (jbf9160@rit.edu)
 */
public class RandomPlayer extends Player {
	
	private Random random = new Random();
	
	/**
	 * Constructor for the RandomPlayer Class.
	 * Simply calls the constructor from the super class with an added
	 * type argument.
	 * 
	 * @param name X or O.
	 */
	public RandomPlayer(String name) {
		super(name, "random");
	}
	
	/**
	 * nextMove decides the next move for the player.
	 * 
	 * @return an integer for which column to place move.
	 */
	public int nextMove(Board board) {
		int c;
		while (true) {
			c = random.nextInt(board.getCol()) * 2 + 1;
			if (! board.fullColumn(c)) {
				return super.nextMove(board, c);
			}
		}
	}

}